let randomClassPrefix = function (length) {
	let result = '';
	let characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
	let charactersLength = characters.length;
	for (let i = 0; i < length; i++) {
		result += characters.charAt(Math.floor(Math.random() * (i > 0 ? charactersLength : charactersLength - 10)));
	}
	return result;
}

let htmlToElement = function (html) {
	var template = document.createElement('template');
	html = html.trim();
	template.innerHTML = html;
	return template.content.firstChild;
}

let copyToClipboard = function (text) {
	try {
		navigator.clipboard.writeText(text).then(
			function() {
				// ok
			}, 
			function() {
				// execCommand устарел в 2022, используем как запасной метод
				var copyFrom = document.createElement('textarea');
				copyFrom.textContent = text;
				document.body.appendChild(copyFrom);
				copyFrom.select();
				document.execCommand('copy');
				copyFrom.blur();
				copyFrom.remove();
			}
		);
	} catch (e) {
		console.error('copyToClipboard:', e);
	}
}

let EVENT_FROM_PAGE = 'event_chrome_ext_' + new Date().getTime();
let EVENT_REPLY = 'event_chrome_ext_reply_' + new Date().getTime();
let PREFIX = randomClassPrefix(10) + '_';

let initCssStyle = function () {
	var css = `.${PREFIX}call_element:hover{ background-color: #e5e5e5; }`;
	var style = document.createElement('style');
	
	if (style.styleSheet) {
		style.styleSheet.cssText = css;
	} else {
		style.appendChild(document.createTextNode(css));
	}
	
	document.getElementsByTagName('head')[0].appendChild(style);
}

let hideAllDropDownMenus = function () {
	let dropDownLists = document.getElementsByClassName(`${PREFIX}drop_down_list`);
	if (dropDownLists.length) {
		for (let i = 0; i < dropDownLists.length; i++) {
			dropDownLists[i].style.display = 'none';
		}
	}
}

let popupPositioning = function (popup) {
	delete popup.style.removeProperty('left');
	let menuLeft = popup.getBoundingClientRect().left; // + document.documentElement.scrollLeft;
	let width = popup.offsetWidth;
	let viewportWidth = window.innerWidth;
	if (menuLeft + width > viewportWidth) {
		popup.style.left = `-${width}px`;
	}

	popup.style.removeProperty('top');
	let menuTop = popup.getBoundingClientRect().top; // + document.documentElement.scrollTop;
	let height = popup.offsetHeight;
	let viewportHeight = window.innerHeight;
	if (menuTop + height > viewportHeight) {
		popup.style.top = `-${height}px`;
	}
}

let alertBlock = function (text, onClose) {
	let style = `
		z-index: 99999;
		position: absolute;
		top: 0px;
		left: 0px;
		background-color: #ffb37d;
		width: 100%;
		text-align: center;
		line-height: 24px;
		color: #000;
		white-space: nowrap;
	`;
	let block = htmlToElement(`
		<div class="${PREFIX}alert_block" style="${style}">
			<div style="padding: 5px 40px; display: inline-block; vertical-align: middle; white-space: normal;">${text}</div>
			<div style="margin-left: -30px; font-size: 26px; display: inline-block; vertical-align: middle; cursor: pointer;" class="${PREFIX}close">✕</div>
		</div>
	`);
	//document.body.insertBefore(block, document.body.firstChild);
	document.body.appendChild(block);
	
	block.querySelectorAll(`.${PREFIX}close`)[0].addEventListener('click', function (e) {
		e = e || window.event;
		let target = e.target || e.srcElement;
		
		target.parentNode.remove();
		
		if (typeof onClose === 'function') {
			try {
				onClose();
			} catch (e) {
				
			}
		}
	});
	
	block.querySelectorAll(`.${PREFIX}link_extensions`)[0].addEventListener('click', function (e) {
		chrome.runtime.sendMessage({
			type: 'openExtensions'
		}, function (response) {
			
		});
	});
}

let somethingWrong = function () {
	return false;
}

initCssStyle();
document.addEventListener('click', function (e) {
	e = e || window.event;
	let target = e.target || e.srcElement;
	
	if (target.className !== `${PREFIX}call_button`) {
		hideAllDropDownMenus();
	}
});
