let initCallButton = function (element, phone) {
	let nPhone = phone.replace(/\D+/g, '');
	if (nPhone.length === 11) {
		if (nPhone[0] === '8') {
			nPhone = '+7' + nPhone.substr(1);
		} else if (nPhone[0] === '7') {
			nPhone = '+' + nPhone;
		}
	}
	
	let callIconUrl = chrome.runtime.getURL('images/call-32x32.png');
	let callBlock = htmlToElement(`
		<div style="display:inline-block; margin-left:15px; position:relative;">\
			<img class="${PREFIX}call_button" src="${callIconUrl}" style="vertical-align: middle; cursor: pointer;">
			<div class="${PREFIX}drop_down_list" style="display:none; position: absolute; z-index: 999999; background-color: white; border: 1px solid #ccc; padding: 2px 2px; font-size: 14px; font-style: normal; font-weight: normal; text-align: left;">
				<div class="${PREFIX}call_element ${PREFIX}copy_to_clipboard" style="white-space: nowrap; line-height: 30px; padding: 0 5px; user-select: none; cursor: pointer;">Копировать номер</div>
				<a href="tel:${nPhone}" style="color: #000; text-decoration: none;"><div class="${PREFIX}call_element" style="white-space: nowrap; line-height: 30px; padding: 0 5px; user-select: none; cursor: pointer;">Позвонить</div></a>
				<div class="${PREFIX}call_element ${PREFIX}pass_to_crm" style="white-space: nowrap; line-height: 30px; padding: 0 5px; user-select: none; cursor: pointer;">Позвонить через CRM</div>
			</div>
		</div>\
	`);
	let dropDownMenu = callBlock.querySelectorAll(`.${PREFIX}drop_down_list`)[0];
	
	callBlock.querySelectorAll('img')[0].addEventListener('click', function (e) {
		e = e || window.event;
		e.stopPropagation();
		e.preventDefault();
		
		hideAllDropDownMenus();
		
		let target = e.target || e.srcElement;
		dropDownMenu.style.display = 'block';
		
		popupPositioning(dropDownMenu);
		
	}, false);
	
	callBlock.querySelectorAll(`.${PREFIX}copy_to_clipboard`)[0].addEventListener('click', function (e) {
		e = e || window.event;
		e.stopPropagation();
		e.preventDefault();
		
		hideAllDropDownMenus();
		
		copyToClipboard(nPhone);
	});
	
	callBlock.querySelectorAll(`.${PREFIX}pass_to_crm`)[0].addEventListener('click', function (e) {
		e = e || window.event;
		e.stopPropagation();
		e.preventDefault();
		
		hideAllDropDownMenus();
		
		chrome.runtime.sendMessage({
			type: 'passPhoneNumberToCRM',
			data: nPhone
		}, function (response) {
			
		});
	});
	
	element.style.whiteSpace = 'nowrap';
	element.style.width = 'auto';
	element.disabled = true;
	element.appendChild(callBlock);
	
	let button = document.querySelectorAll('[data-test="OfferCardSummary"] [class^="OfferCardContacts__container"] button[class*="OfferCardContacts__phonesButton--"]');
	if (button.length) {
		button[0].style.overflow = 'visible';
		if (button[0].children.length) {
			button[0].children[0].style.overflow = 'visible';
		}
		let qrCodes = button[0].querySelectorAll('[class*="OfferCardContacts__qrCode--"]');
		if (qrCodes.length) {
			qrCodes[0].style.top = '0px';
		}
	}
}

let createObserver = function (options) {
	let selectorHidden = options.phoneElem.hidden;
	let selectorShown = options.phoneElem.shown;
	
	if (selectorHidden.constructor === Array) {
		selectorHidden = selectorHidden.join(',');
	}
	if (selectorShown.constructor === Array) {
		selectorShown = selectorShown.join(',');
	}
	
	let callButtons = document.querySelectorAll(selectorHidden);
	if (callButtons.length > 0) {
		var observer = new MutationObserver(function (mutations) {
			let phones = document.querySelectorAll(selectorShown);
			if (phones.length > 0) {
				for (let i = 0; i < phones.length; i++) {
					let phone = phones[i];
					if (phone.hasOwnProperty('crmReady')) continue;
					phone.crmReady = true;
					if (phone.querySelectorAll(`.${PREFIX}call_button`).length > 0) continue;
					
					if (options.skip && options.skip.length) {
						let skipFired = false;
						for (let c = 0; c < options.skip.length; c++) {
							skipFired = skipFired || options.skip[c](phone);
						}
						if (skipFired) continue;
					}
					
					let placeToElem = phone;
					if (options.icon && options.icon.preprocessElem && typeof options.icon.preprocessElem === 'function') {
						placeToElem = options.icon.preprocessElem(placeToElem);
					}
					
					initCallButton(placeToElem, phone[ options.phoneElem.property ], options.icon.isMini, options.icon.style);
					
					if (options.icon && options.icon.postprocessElem && typeof options.icon.postprocessElem === 'function') {
						options.icon.postprocessElem(placeToElem);
					}
				}
			}
		});
		
		let observerOptions = options.observerOptions || { childList: true, subtree: true };
		observer.observe(document.body, observerOptions);
	} else {
		if (typeof options.isWrong === 'function' && options.isWrong()) {
			somethingWrong();
		}
	}
}

let replacments = [{
	// Страница объявления
	'observerOptions': { childList: true, subtree: true, attributes: true },
	'phoneElem': {
		'hidden': '.Page.Page_type_offer',
		'shown': `[data-test="OfferCardSummary"] [class^="OfferCardContacts__container"] button[class*="OfferCardContacts__phonesButton--"] a[data-test="OfferCardContactsPhoneShown"]:not(.${PREFIX}call_a_element)`,
		'property': 'href'
	},
	'icon': {
		'isMini': false,
		'style': 'margin-left:-10px; margin-top: 4px;',
		'preprocessElem': false
	},
	'skip': [
		function (elem) { return (elem.style.whiteSpace === 'nowrap'); },
		function (elem) { return (elem.textContent.indexOf('Показать') != -1); }
	],
	'isWrong': false
}, {
	// Страница списка объявлений
	'observerOptions': { childList: true, subtree: true },
	'phoneElem': {
		'hidden': '[class="Portal"]',
		'shown': [
			`.PhoneModal__content [data-test="PhoneModalContactsPhoneShown"]:not(.${PREFIX}call_a_element)`,
			`.PhoneModalContacts__phone [data-test="PhoneModalContactsPhoneShown"]:not(.${PREFIX}call_a_element)`
		],
		'property': 'textContent'
	},
	'icon': {
		'isMini': false,
		'preprocessElem': false,
		'postprocessElem': function (elem) {
			elem.querySelectorAll(`img.${PREFIX}call_button`)[0].style.verticalAlign = 'top';
		}
	},
	'skip': [
		function (elem) { return (elem.style.whiteSpace === 'nowrap'); },
		function (elem) { return (elem.textContent.indexOf('Показать') != -1); }
	],
	'isWrong': false
}];

function checkAndInitObservers() {
	if (document.querySelectorAll('[data-test="OfferCardSummary"] [class^="OfferCardContacts__container"]').length || document.querySelectorAll('[class*="__list-item_type_offer"]').length) {
		replacments.forEach(a => createObserver(a));
	} else {
		setTimeout(checkAndInitObservers, 1000);
	}
}
setTimeout(checkAndInitObservers, 1000);
